//+------------------------------------------------------------------+
//|                                                         awal.mq5 |
//|                                                            frizt |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "frizt"
#property link      "https://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
#property strict
#include <Trade\Trade.mqh>

input int      StochasticPeriodK = 14;
input int      StochasticPeriodD = 3;
input int      StochasticSlowing = 3;
input ENUM_STO_PRICE StochasticPrice = STO_LOWHIGH;

input bool     UseTrendFilter = true;
input int      MAPeriod = 200;
input ENUM_MA_METHOD MAMethod = MODE_SMA;
input ENUM_APPLIED_PRICE MAPrice = PRICE_CLOSE;

input double   StopLossPoints = 300;
input double   TakeProfitPoints = 500;
input double   FixedLotSize = 0.1;
input bool     UsePartialClose = true;
input int      PartialCloseLevel1 = 30;
input int      PartialCloseLevel2 = 50;
input ulong    MagicNumber = 2024;

CTrade trade;
int handleStoch, handleMA;
double StochK[], StochD[], MA[];
datetime lastBar = 0;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnInit()
{
   handleStoch = iStochastic(_Symbol, _Period, StochasticPeriodK, StochasticPeriodD, StochasticSlowing, MAMethod, StochasticPrice);
   handleMA = iMA(_Symbol, _Period, MAPeriod, 0, MAMethod, MAPrice);
   if (handleStoch == INVALID_HANDLE || handleMA == INVALID_HANDLE)
   {
      Print("Indicator initialization failed");
      return INIT_FAILED;
   }
   ArraySetAsSeries(StochK, true);
   ArraySetAsSeries(StochD, true);
   ArraySetAsSeries(MA, true);
   return INIT_SUCCEEDED;
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnTick()
{
   if (!IsNewBar()) return;

   if (CopyBuffer(handleStoch, 0, 0, 3, StochK) <= 0 ||
       CopyBuffer(handleStoch, 1, 0, 3, StochD) <= 0 ||
       CopyBuffer(handleMA, 0, 0, 3, MA) <= 0)
      return;

   double bid = SymbolInfoDouble(_Symbol, SYMBOL_BID);
   double ask = SymbolInfoDouble(_Symbol, SYMBOL_ASK);

   bool buySignal = (StochK[1] < StochD[1]) && (StochK[0] > StochD[0]);
   bool sellSignal = (StochK[1] > StochD[1]) && (StochK[0] < StochD[0]);

   if (UseTrendFilter)
   {
      buySignal &= (ask > MA[0]);
      sellSignal &= (bid < MA[0]);
   }

   if (buySignal && !HasOpenPosition(POSITION_TYPE_BUY))
   {
      double sl = ask - StopLossPoints * _Point;
      double tp = ask + TakeProfitPoints * _Point;
      trade.Buy(NormalizeLotSize(FixedLotSize), _Symbol, ask, sl, tp, "Buy");
   }

   if (sellSignal && !HasOpenPosition(POSITION_TYPE_SELL))
   {
      double sl = bid + StopLossPoints * _Point;
      double tp = bid - TakeProfitPoints * _Point;
      trade.Sell(NormalizeLotSize(FixedLotSize), _Symbol, bid, sl, tp, "Sell");
   }

   if (UsePartialClose)
      PartialClose();
}

//+------------------------------------------------------------------+
//| Partial close management                                         |
//+------------------------------------------------------------------+
void PartialClose()
{
   for (int i = PositionsTotal() - 1; i >= 0; i--)
   {
      if (PositionGetTicket(i) > 0 && PositionGetInteger(POSITION_MAGIC) == (long)MagicNumber)
      {
         double openPrice = PositionGetDouble(POSITION_PRICE_OPEN);
         double currentPrice = (PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY) ? 
                                SymbolInfoDouble(_Symbol, SYMBOL_BID) : 
                                SymbolInfoDouble(_Symbol, SYMBOL_ASK);
         double profitPoints = MathAbs(currentPrice - openPrice) / _Point;
         double currentLots = PositionGetDouble(POSITION_VOLUME);

         if (profitPoints >= PartialCloseLevel1 && currentLots > 0.01)
         {
            trade.PositionClosePartial(PositionGetInteger(POSITION_TICKET), currentLots / 2);
         }
      }
   }
}

//+------------------------------------------------------------------+
//| Check if there's a new candle                                    |
//+------------------------------------------------------------------+
bool IsNewBar()
{
   datetime time = iTime(_Symbol, _Period, 0);
   if (time != lastBar)
   {
      lastBar = time;
      return true;
   }
   return false;
}

//+------------------------------------------------------------------+
//| Check open positions                                             |
//+------------------------------------------------------------------+
bool HasOpenPosition(ENUM_POSITION_TYPE type)
{
   for (int i = PositionsTotal() - 1; i >= 0; i--)
   {
      if (PositionGetTicket(i) > 0 && PositionGetInteger(POSITION_TYPE) == type &&
          PositionGetInteger(POSITION_MAGIC) == (long)MagicNumber)
         return true;
   }
   return false;
}

//+------------------------------------------------------------------+
//| Normalize lot size                                               |
//+------------------------------------------------------------------+
double NormalizeLotSize(double lot)
{
   double minLot = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_MIN);
   double maxLot = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_MAX);
   double lotStep = SymbolInfoDouble(_Symbol, SYMBOL_VOLUME_STEP);
   lot = MathMax(MathMin(lot, maxLot), minLot);
   return MathFloor(lot / lotStep) * lotStep;
}
